@@ -1,9 +1,9 @@
-from collections import defaultdict
 import formencode
+import inspect
 from blazeutils.datastructures import LazyOrderedDict
 
 from blazeform.element import form_elements, CancelElement, CheckboxElement, \
-        MultiSelectElement, LogicalGroupElement
+    MultiSelectElement, LogicalGroupElement
 from blazeform.exceptions import ElementInvalid, ProgrammingError
 from blazeform.file_upload_translators import WerkzeugTranslator
 from blazeform.processors import Wrapper
@@ -63,24 +63,25 @@ class FormBase(HtmlAttributeHolder, ElementRegistrar):
         for el in self.els.values():
             if el.is_submittable:
                 yield el
+
     @property
     def renderable_els(self):
         for el in self.els.values():
             if el.is_renderable and not el.renders_in_group:
                 yield el
+
     @property
     def returning_els(self):
         for el in self.els.values():
             if el.is_returning:
                 yield el
 
-
     def register_elements(self, dic):
         for type, eclass in dic.items():
             self.register_element_type(type, eclass)
 
     def register_element_type(self, type, eclass):
-        if self._registered_types.has_key(type):
+        if type in self._registered_types:
             raise ValueError('type "%s" is already registered' % type)
         self._registered_types[type] = eclass
 
@@ -117,7 +118,7 @@ class FormBase(HtmlAttributeHolder, ElementRegistrar):
                     return True
         return False
 
-    def add_validator(self, validator, msg = None):
+    def add_validator(self, validator, msg=None):
         """
             form level validators are only validators, no manipulation of
             values can take place.  The validator should be a formencode
@@ -135,6 +136,12 @@ class FormBase(HtmlAttributeHolder, ElementRegistrar):
                 validator = Wrapper(to_python=validator)
             else:
                 raise TypeError('validator must be a Formencode validator or a callable')
+        else:
+            # FE validators may be passed as the class or an instance
+            #   if class, then make it an instance
+            if inspect.isclass(validator):
+                validator = validator()
+
         self._validators.append((validator, msg))
 
     def add_field_errors(self, errors):
@@ -147,7 +154,8 @@ class FormBase(HtmlAttributeHolder, ElementRegistrar):
                     for error in errors[el]:
                         getattr(self.elements, el).errors.append(error)
                 else:
-                    raise TypeError('add_field_errors must be passed a dictionary with values of either strings, or lists of strings')
+                    raise TypeError('add_field_errors must be passed a dictionary with '
+                                    'values of either strings, or lists of strings')
                 del errors[el]
         # indicate that some errors were not added
         if errors:
@@ -167,13 +175,13 @@ class FormBase(HtmlAttributeHolder, ElementRegistrar):
         # whole form validation
         for validator, msg in self._validators:
             try:
-                value = validator.to_python(self)
-            except formencode.Invalid, e:
+                validator.to_python(self)
+            except formencode.Invalid as e:
                 valid = False
                 msg = (msg or str(e))
                 if msg:
                     self.add_error(msg)
-            except ElementInvalid, e:
+            except ElementInvalid:
                 # since we are getting an ElementInvalid exception, that means
                 # our validator needed the value of an element to complete
                 # validation, but that element is invalid.  In that case,
@@ -183,6 +191,14 @@ class FormBase(HtmlAttributeHolder, ElementRegistrar):
 
         return valid
 
+    def _set_submitted_values(self, values):
+        for el in self.submittable_els:
+            key = el.nameattr or el.id
+            if key in values:
+                el.submittedval = values[key]
+            elif isinstance(el, (CheckboxElement, MultiSelectElement, LogicalGroupElement)):
+                el.submittedval = None
+
     def set_submitted(self, values):
         """ values should be dict like """
 
@@ -196,20 +212,15 @@ class FormBase(HtmlAttributeHolder, ElementRegistrar):
         # apply the submitted values
         identel = getattr(self.elements, self._form_ident_field)
         ident_key = identel.nameattr or identel.id
-        if values.has_key(ident_key):
+        if ident_key in values:
             identel.submittedval = values[ident_key]
 
         if self._is_submitted():
-            for el in self.submittable_els:
-                key = el.nameattr or el.id
-                if values.has_key(key):
-                    el.submittedval = values[key]
-                elif isinstance(el, (CheckboxElement, MultiSelectElement, LogicalGroupElement)):
-                        el.submittedval = None
+            self._set_submitted_values(values)
 
     def set_defaults(self, values):
         for el in self.defaultable_els:
-            if values.has_key(el.id):
+            if el.id in values:
                 el.defaultval = values[el.id]
 
     def get_values(self):
@@ -224,7 +235,8 @@ class FormBase(HtmlAttributeHolder, ElementRegistrar):
         return retval
     values = property(get_values)
 
-    def add_handler(self, exception_txt=NotGiven, error_msg=NotGiven, exc_type=NotGiven, callback=NotGiven):
+    def add_handler(self, exception_txt=NotGiven, error_msg=NotGiven, exc_type=NotGiven,
+                    callback=NotGiven):
         self._exception_handlers.append((exception_txt, error_msg, exc_type, callback))
 
     def handle_exception(self, exc):
@@ -242,7 +254,7 @@ class FormBase(HtmlAttributeHolder, ElementRegistrar):
 
         for looking_for, error_msg, exc_type, callback in self._exception_handlers:
             if not is_notgiven(exc_type):
-                if isinstance(exc_type, basestring):
+                if isinstance(exc_type, str):
                     if exc.__class__.__name__ != exc_type:
                         continue
                 else:
@@ -280,6 +292,7 @@ class FormBase(HtmlAttributeHolder, ElementRegistrar):
                 field_errors[key].append(msg)
         return form_errors, field_errors
 
+
 class Form(FormBase):
     """
     Main form class using default HTML renderer and Werkzeug file upload
@@ -287,7 +300,7 @@ class Form(FormBase):
     """
     def __init__(self, name, static=False, **kwargs):
         # make the form's name the id
-        if not kwargs.has_key('id'):
+        if 'id' not in kwargs:
             kwargs['id'] = name
 
         FormBase.__init__(self, name, static, **kwargs)
